SystemOrganization addCategory: #'ToolBuilder-Morphic'!

ToolBuilder subclass: #MorphicToolBuilder
	instanceVariableNames: 'widgets panes parentMenu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ToolBuilder-Morphic'!

!MorphicToolBuilder commentStamp: 'ar 2/11/2005 15:02' prior: 0!
The Morphic tool builder.!

!MorphicToolBuilder class methodsFor: 'accessing' stamp: 'alain.plantec 5/30/2008 13:55'!
isActiveBuilder
	"Answer whether I am the currently active builder"
	^ true! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 7/17/2005 00:00'!
add: aMorph to: aParent
	aParent addMorphBack: aMorph.
	aParent isSystemWindow ifTrue:[
		aParent addPaneMorph: aMorph.
	].! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'stephaneducasse 2/3/2006 22:35'!
asFrame: aRectangle
	| frame |
	aRectangle ifNil:[^nil].
	frame := LayoutFrame new.
	frame 
		leftFraction: aRectangle left; 
		rightFraction: aRectangle right; 
		topFraction: aRectangle top; 
		bottomFraction: aRectangle bottom.
	^frame! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'btr 11/26/2006 12:49'!
buildHelpFor: widget spec: aSpec
	aSpec help
		ifNotNil: [widget setBalloonText: aSpec help]! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 8/7/2009 19:40'!
buildPluggableActionButton: aSpec
	| button |
	button := self buildPluggableButton: aSpec.
	button color: Color white.
	^button! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'StephaneDucasse 7/31/2010 19:21'!
buildPluggableButton: aSpec

	| widget label state action enabled |
	label := aSpec label.
	state := aSpec state.
	action := aSpec action.
	widget := self buttonClass 
					on: aSpec model
 					getState: (state isSymbol ifTrue: [state])
  					action: nil
 					label: (label isSymbol ifTrue: [label]).
	self register: widget id: aSpec name.
	enabled := aSpec enabled.
	enabled isSymbol
		ifTrue: [widget getEnabledSelector: enabled]
		ifFalse: [widget enabled:enabled].
	action isSymbol
		ifTrue: [widget action: action]
		ifFalse: [widget actionBlock: action].
	widget getColorSelector: aSpec color.
	widget offColor: Color white..
	self buildHelpFor: widget spec: aSpec.
	(label isSymbol or: [label == nil]) ifFalse: [widget label: label].
	self setFrame: aSpec frame in: widget.
	parent ifNotNil: [self add: widget to: parent].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'AlainPlantec 9/22/2011 20:52'!
buildPluggableCodePane: aSpec 
	^ self buildPluggableText: aSpec! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'md 8/15/2005 17:55'!
buildPluggableInputField: aSpec
	| widget |
	widget := self buildPluggableText: aSpec.
	widget acceptOnCR: true.
	widget hideScrollBarsIndefinitely.
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 1/23/2010 14:32'!
buildPluggableList: aSpec
	| widget listClass getIndex setIndex |
	aSpec getSelected ifNil:[
		listClass := self listClass.
		getIndex := aSpec getIndex.
		setIndex := aSpec setIndex.
	] ifNotNil:[
		listClass := self listByItemClass.
		getIndex := aSpec getSelected.
		setIndex := aSpec setSelected.
	].
	widget := listClass on: aSpec model
				list: aSpec list
				selected: getIndex
				changeSelected: setIndex
				menu: aSpec menu
				keystroke: aSpec keyPress.
	self register: widget id: aSpec name.
	widget getListElementSelector: aSpec listItem.
	widget getListSizeSelector: aSpec listSize.
	widget doubleClickSelector: aSpec doubleClick.
	widget dragItemSelector: aSpec dragItem.
	widget dropItemSelector: aSpec dropItem.
	widget wantsDropSelector: aSpec dropAccept.
	widget autoDeselect: aSpec autoDeselect.
	self buildHelpFor: widget spec: aSpec. 
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[self add: widget to: parent].
	panes ifNotNil:[
		aSpec list ifNotNil:[panes add: aSpec list].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'tfel 1/8/2012 14:19'!
buildPluggableMultiSelectionList: aSpec
	| widget listClass |
	aSpec getSelected ifNotNil:[^self error:'There is no PluggableListMorphOfManyByItem'].
	listClass := self listClass.
	widget := listClass on: aSpec model
		list: aSpec list
		primarySelection: aSpec getIndex
		changePrimarySelection: aSpec setIndex
		listSelection: aSpec getSelectionList
		changeListSelection: aSpec setSelectionList
		menu: aSpec menu.
	widget beMultipleSelection.
	self register: widget id: aSpec name.
	widget keystrokeActionSelector: aSpec keyPress.
	widget getListElementSelector: aSpec listItem.
	widget getListSizeSelector: aSpec listSize.
	self buildHelpFor: widget spec: aSpec. 
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[self add: widget to: parent].
	panes ifNotNil:[
		aSpec list ifNotNil:[panes add: aSpec list].
	].
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 1/23/2010 14:33'!
buildPluggablePanel: aSpec
	| widget children frame |
	widget := self panelClass new.
	self register: widget id: aSpec name.
	widget model: aSpec model.
	widget color: Color transparent.
	widget clipSubmorphs: true.
	children := aSpec children.
	children isSymbol ifTrue:[
		widget getChildrenSelector: children.
		widget update: children.
		children := #().
	].
	self buildAll: children in: widget.
	self buildHelpFor: widget spec: aSpec. 
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[self add: widget to: parent].
	self setLayout: aSpec layout in: widget.
	widget layoutInset: 0.
	widget borderWidth: 0.
	widget submorphsDo:[:sm|
		(frame := sm layoutFrame) ifNotNil:[
			(frame rightFraction = 0 or:[frame rightFraction = 1]) 
				ifFalse:[frame rightOffset:1].
			(frame bottomFraction = 0 or:[frame bottomFraction = 1]) 
				ifFalse:[frame bottomOffset: 1]]].
	widget color: Color transparent.
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'AlainPlantec 9/22/2011 20:51'!
buildPluggableText: aSpec 
	| widget |
	widget := self textPaneClass
				on: aSpec model
				text: aSpec getText
				accept: aSpec setText
				readSelection: aSpec selection 
				menu: aSpec menu.
	widget askBeforeDiscardingEdits: aSpec askBeforeDiscardingEdits.
	widget font: StandardFonts codeFont.
	self register: widget id: aSpec name.
	widget getColorSelector: aSpec color.
	self buildHelpFor: widget spec: aSpec.
	self setFrame: aSpec frame in: widget.
	parent
		ifNotNil: [self add: widget to: parent].
	widget borderColor: Color lightGray.
	widget color: Color white.
	^ widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 1/23/2010 14:33'!
buildPluggableTree: aSpec
	| widget |
	widget := self treeClass new.
	self register: widget id: aSpec name.
	widget model: aSpec model.
	widget getSelectedPathSelector: aSpec getSelectedPath.
	widget setSelectedSelector: aSpec setSelected.
	widget getChildrenSelector: aSpec getChildren.
	widget hasChildrenSelector: aSpec hasChildren.
	widget getLabelSelector: aSpec label.
	widget getIconSelector: aSpec icon.
	widget getHelpSelector: aSpec help.
	widget getMenuSelector: aSpec menu.
	widget keystrokeActionSelector: aSpec keyPress.
	widget getRootsSelector: aSpec roots.
	widget autoDeselect: aSpec autoDeselect.
	widget dropItemSelector: aSpec dropItem.
	widget wantsDropSelector: aSpec dropAccept.
	widget dragItemSelector: aSpec dragItem.
	self setFrame: aSpec frame in: widget.
	parent ifNotNil:[self add: widget to: parent].
"	panes ifNotNil:[
		aSpec roots ifNotNil:[panes add: aSpec roots].
	].	"
	^widget! !

!MorphicToolBuilder methodsFor: 'pluggable widgets' stamp: 'ar 1/23/2010 14:33'!
buildPluggableWindow: aSpec
	| widget children label |
	aSpec layout == #proportional ifFalse:[
		"This needs to be implemented - probably by adding a single pane and then the rest"
		^self error: 'Not implemented'.
	].
	widget := self windowClass new.
	self register: widget id: aSpec name.
	widget model: aSpec model.
	(label := aSpec label) ifNotNil:[
		label isSymbol 
			ifTrue:[widget getLabelSelector: label]
			ifFalse:[widget setLabel: label]].
	children := aSpec children.
	children isSymbol ifTrue:[
		widget getChildrenSelector: children.
		widget update: children.
		children := #().
	].
	widget closeWindowSelector: aSpec closeAction.
	panes := OrderedCollection new.
	self buildAll: children in: widget.
	self buildHelpFor: widget spec: aSpec. 
	widget bounds: (RealEstateAgent 
		initialFrameFor: widget 
		initialExtent: (aSpec extent ifNil:[widget initialExtent])
		world: self currentWorld).
	widget setUpdatablePanesFrom: panes.
	^widget! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'MarcusDenker 7/29/2010 20:37'!
buttonClass
	^ PluggableButtonMorph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:40'!
close: aWidget
	"Close a previously opened widget"
	aWidget delete! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'stephaneDucasse 7/18/2010 12:03'!
codePaneClass
	^ PluggableTextMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'stephaneDucasse 7/18/2010 11:53'!
listByItemClass
	^ PluggableListMorphByItem! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'StephaneDucasse 7/18/2010 13:43'!
listClass
	^ PluggableListMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:20'!
menuClass
	^ MenuMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:20'!
menuItemClass
	^ MenuItemMorph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 10/4/2009 22:47'!
open: anObject
	"Build and open the object. Answer the widget opened."
	| morph |
	anObject isMorph 
		ifTrue:[morph := anObject]
		ifFalse:[morph := self build: anObject].
	(morph isKindOf: MenuMorph)
		ifTrue:[morph popUpInWorld: World].
	(morph isKindOf: SystemWindow)
		ifTrue:[morph openInWorldExtent: morph extent]
		ifFalse:[morph openInWorld].
	^morph! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:40'!
open: anObject label: aString
	"Build an open the object, labeling it appropriately.  Answer the widget opened."
	| window |
	window := self open: anObject.
	window setLabel: aString.
	^window! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:14'!
panelClass
	^ PluggablePanelMorph! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 9/6/2009 16:14'!
register: widget id: id
	id ifNil:[^self].
	widgets ifNil:[widgets := Dictionary new].
	widgets at: id put: widget.
	widget setNameTo: id.! !

!MorphicToolBuilder methodsFor: 'opening' stamp: 'ar 6/5/2005 12:41'!
runModal: aWidget
	"Run the (previously opened) widget modally, e.g., 
	do not return control to the sender before the user has responded."
	[aWidget world notNil] whileTrue: [
		aWidget outermostWorldMorph doOneCycle.
	].
! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'dc 1/10/2008 14:17'!
setFrame: aRectangle in: widget
	| frame |
	aRectangle ifNil:[^nil].
	frame := aRectangle isRectangle
		ifTrue: [self asFrame: aRectangle]
		ifFalse: [aRectangle]. "assume LayoutFrame"
	widget layoutFrame: frame.
	widget hResizing: #spaceFill; vResizing: #spaceFill.
	(parent isSystemWindow) ifTrue:[
		widget borderWidth: 2; borderColor: #inset.
	].! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 9/16/2009 23:11'!
setLayout: layout in: widget
	layout == #proportional ifTrue:[
		widget layoutPolicy: ProportionalLayout new.
		^self].
	layout == #horizontal ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #leftToRight.
		widget submorphsDo:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
		widget cellInset: 1@1.
		widget layoutInset: 1@1.
		widget color: Color transparent.
		"and then some..."
		^self].
	layout == #vertical ifTrue:[
		widget layoutPolicy: TableLayout new.
		widget listDirection: #topToBottom.
		widget submorphsDo:[:m| m hResizing: #spaceFill; vResizing: #spaceFill].
		widget cellInset: 1@1.
		widget layoutInset: 1@1.
		widget color: Color transparent.
		"and then some..."
		^self].
	^self error: 'Unknown layout: ', layout.! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'StephaneDucasse 7/17/2010 15:44'!
textPaneClass
	^ PluggableTextMorph! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:16'!
treeClass
	^ PluggableTreeMorph! !

!MorphicToolBuilder methodsFor: 'private' stamp: 'ar 7/14/2005 22:30'!
widgetAt: id ifAbsent: aBlock
	widgets ifNil:[^aBlock value].
	^widgets at: id ifAbsent: aBlock! !

!MorphicToolBuilder methodsFor: 'widget classes' stamp: 'Igor.Stasenko 1/17/2010 04:16'!
windowClass
	^ PluggableSystemWindow! !
